/**
	Segdosztly BMP kpek beolvassra, megjelentsre s kirsra.

	Hasznlat:
		A struktra adattgjai trolja a kp mreteit s a bitek szneit.

		Adattagok:
		- x, y:		A kp szlessge s magassga
		- pixels:	Mutat a kppont szneket trol tmbre. Minden pixel egy
					RGBA byte-ngyessel van lerva amik a kpont szncsatornin
					az intenzitsokat troljk

		Mkds: Az osztly metdusain keresztl.
		- BitmapStruct(int width, int height) :
			res kp ltrehozsa adott mrettel.

		- BitmapStruct(char* filename) :
			Kp beolvassa filebl

		- int writeFile(char* filename) :
			Az aktulisan trolt kp kirsa fileba

		- void displayImage() :
			Aktulisan trolt kp megjelentse egy ablakba, majd kilps a programbl.


	Ksztette: Varga Lszl

 */

#include "imageIO.h"
#include <stdlib.h>

#ifndef __BITMAPSTRUCT_H___
#define __BITMAPSTRUCT_H___

#  ifdef __cplusplus
extern "C" {
#  endif /* __cplusplus */


struct BitmapStruct {
	unsigned char* pixels;
	int x, y;

	BitmapStruct(int width, int height) {
		pixels = new unsigned char[width * height * 4];
		x = width;
		y = height;
	}

	BitmapStruct(char* filename) {
		int i;
		int src_index = 0;
		int dst_index = 0;

		unsigned char* temp_img = NULL;

		loadBmp(filename, &x, &y, &temp_img);

		pixels = new unsigned char[x * y * 4];

		for(i=0, src_index=0, dst_index=0; i < x*y; i++, src_index+=3, dst_index+=4) {
			pixels[dst_index + 0] = temp_img[src_index + 2];
			pixels[dst_index + 1] = temp_img[src_index + 1];
			pixels[dst_index + 2] = temp_img[src_index + 0];
		}

		delete[] temp_img;
	}

	~BitmapStruct() {
		delete[] pixels;
	}

	unsigned char* get_ptr(void) const {
		return pixels;
	}

	long image_size( void ) const {
		return x * y * 4;
	}

/*	int loadFile(char* filename) {
		int i;
		int src_index = 0;
		int dst_index = 0;

		unsigned char* temp_img = NULL;

		loadBmp(filename, &x, &y, &temp_img);

		for(i=0, src_index=0, dst_index=0; i < x*y; i++, src_index+=3, dst_index+=4) {
			pixels[dst_index + 0] = temp_img[src_index + 2];
			pixels[dst_index + 1] = temp_img[src_index + 1];
			pixels[dst_index + 2] = temp_img[src_index + 0];
		}

		delete[] temp_img;
	}*/

	int writeFile(char* filename) {
		int i;
		int src_index = 0;
		int dst_index = 0;

		unsigned char* temp_img = new unsigned char[x * y * 3];

		for(i=0, src_index=0, dst_index=0; i < x*y; i++, src_index+=4, dst_index+=3) {
			temp_img[dst_index + 0] = pixels[src_index + 2];
			temp_img[dst_index + 1] = pixels[src_index + 1];
			temp_img[dst_index + 2] = pixels[src_index + 0];
		}

		writeBmp(filename, x, y, temp_img);

		delete[] temp_img;

		return 0;
	}

	static BitmapStruct** getBitmapPtr( void ){
		static BitmapStruct* ptr;
		return &ptr;
	}

	static void Key(unsigned char key, int x, int y) {
		switch(key) {
			case 27:
				exit(0);
		}

	}

	static void Draw() {
		BitmapStruct* bitmapPtr = *(getBitmapPtr());

		glClearColor(0.0, 0.0, 0.0, 1.0);
		glClear(GL_COLOR_BUFFER_BIT);
		glDrawPixels(bitmapPtr->x, bitmapPtr->y, GL_RGBA, GL_UNSIGNED_BYTE, bitmapPtr->pixels);
		glFlush();

	}

	void displayImage() {
		BitmapStruct** bitmapPtr = getBitmapPtr();
		*bitmapPtr = this;

		int c = 1;
		char* str = "";
		
		glutInit(&c, &str);
		glutInitDisplayMode(GLUT_SINGLE || GLUT_RGBA);
		glutInitWindowSize( x, y );
		glutCreateWindow("bitmap image");
		glutKeyboardFunc(Key);
		glutDisplayFunc(Draw);
		glutMainLoop();

		return;
	}

};


#  ifdef __cplusplus
}
#  endif /* __cplusplus */


#endif /* __BITMAPSTRUCT_H___ */
