#include "BitmapIO.h"

#ifndef BMPCORR_H
#define BMPCORR_H

#  ifdef __cplusplus
extern "C" {
#  endif /* __cplusplus */

void bmpunpad(void** unpadded,
			  void* padded,
			  const BITMAPINFO* info);

void bmppad(void** padded,
			void* unpadded,
			const BITMAPINFO* info);

void bmpunindex(void** pixels,
				void* input,
				const BITMAPINFO* info);


void bmpunpad(void** unpadded,
			  void* padded,
			  const BITMAPINFO* info)
{
	int i;
	unsigned char* src_start;
	unsigned char* dst_start;
	void* output;
	int byteperpixel;
	int padding;
	int row_length;

	byteperpixel = (info->bmiHeader.biBitCount + 7) / 8;
	padding = (4 - (info->bmiHeader.biWidth*byteperpixel) % 4) % 4;
	row_length = info->bmiHeader.biWidth*byteperpixel;

	if(*unpadded == NULL)
	{
		*unpadded = (void*)malloc(info->bmiHeader.biHeight * row_length);
	}
	output = *unpadded;

	for(i=0; i<info->bmiHeader.biHeight; i++)
	{
		src_start = (unsigned char*)padded + i * (row_length + padding);
		dst_start = (unsigned char*)output + i * row_length;
		memcpy(dst_start, src_start, row_length);
	}

	return;
}

void bmppad(void** padded,
			void* unpadded,
			const BITMAPINFO* info)
{
	int i;
	unsigned char* src_start;
	unsigned char* dst_start;
	void* output;
	int byteperpixel;
	int padding;
	int row_length;

	byteperpixel = (info->bmiHeader.biBitCount + 7) / 8;
	padding = (4 - (info->bmiHeader.biWidth*byteperpixel) % 4) % 4;
	row_length = info->bmiHeader.biWidth*byteperpixel;

	if(*padded == NULL)
	{
		*padded = (void*)malloc(info->bmiHeader.biHeight * (row_length + padding));
	}

	output = *padded;

	for(i=info->bmiHeader.biHeight-1; i>=0; i--)
	{
		dst_start = (unsigned char*)output + i * (row_length + padding);
		src_start = (unsigned char*)unpadded + i * row_length;
		memcpy(dst_start, src_start, row_length);

		memset(dst_start + row_length, 0, padding);
	}

	return;
}

void bmpunindex(void** pixels,
				void* input,
				const BITMAPINFO* info)
{
	int i, j, k;

	unsigned char* ptr;

	int bit_per_pixel = info->bmiHeader.biBitCount;
	int pixel_per_byte = 8 / bit_per_pixel;

	int byte_index;
	int offset_in_byte;
	unsigned char mask;

	int out_index;

	unsigned char color_index;

	int byte_per_row = info->bmiHeader.biSizeImage / info->bmiHeader.biHeight;

	mask = (1 << bit_per_pixel) - 1;

	ptr = (unsigned char*)malloc(info->bmiHeader.biWidth * info->bmiHeader.biHeight * 3 * sizeof(unsigned char));
	*pixels = ptr;

	for(i=0; i<info->bmiHeader.biHeight; i++) {
		for(j=0; j<info->bmiHeader.biWidth; j++) {

			byte_index = i*byte_per_row + j / pixel_per_byte;
			offset_in_byte = (pixel_per_byte - (j % pixel_per_byte) - 1) * bit_per_pixel;

			out_index = (i * info->bmiHeader.biWidth + j) * 3;

			color_index = (((unsigned char*)input)[byte_index] >> offset_in_byte) & mask;

			ptr[out_index + 0] = info->bmiColors[color_index].rgbRed;
			ptr[out_index + 1] = info->bmiColors[color_index].rgbGreen;
			ptr[out_index + 2] = info->bmiColors[color_index].rgbBlue;
//			ptr[out_index + 0] = 255;
//			ptr[out_index + 1] = 255;
//			ptr[out_index + 2] = 255;
		}
	}

	return;
}




#  ifdef __cplusplus
}
#  endif /* __cplusplus */

#endif /* BMPCORR_H */
